# perl itxsed.pl <file>.s
# 
# Pre-process ITRANS encoded text files (like 1.s isb song files) for use
# with xdvng html output. 
# xdvng does not have punctuation, so it has to be escaped (using ##..##, which
# gets processed by itrans itself).
# Assumes a \stitle, \printtitle etc in input,
# outputs </PRE><PRE class=xdvng-eng>
#
# Special handling:
# \stitle{...text...} has text that is assumed to be enclosed with
# #indian ... text ... #endindian, and can contain ##, which needs
# to be disabled.
# % text comment lines need to sent unchanged from this filter.
# Avinash Chopde <avinash@acm.org> 2/11/00
# 
# Example input file:
# % 
# \startsong
# \stitle{jogii jabase tuu aayaa mere dvaare}%
# \film{Bandini}%
# \starring{Ashok Kumar, Nutan}%
# \singer{Lata}%
# \music{S D Burman}%
# \lyrics{Gulzar}%
# % 
# % Contributor: Himanshu Gupta (himanshu.gupta@eng.sun.com)
# % Credits:  
# % Transliterator: Rajiv Shridhar (rajiv@hendrix.coe.neu.edu)
# %
# \printtitle
# #indian
# %
# jogii jabase tuu aayaa mere dvaare
# mere ra.Ng ga_e saa.njh sakaare
# tuu to a.Nkhiyo.n me.n jaan\-e\-jii kii batiyaa.N
# % some comment here
# tujhase milanaa hii zulm bhaayaa re
# o jogii jabase \threedots
# ... 
# %
# #endindian
# \endsong
# %

#-------------------------------------------------------------------------
# This is the main subroutine that performs substitutions on the
# itrans encoded text.
# Changes here include:
#    punctuation, replaced/protected for Xdvng
# Add more substititions as required here

sub itxsubstitute {
    local($lword) = @_;
    # $lword is now a string that was found between #indian .. #endindian

    # remove any \'s left - this will be \-, \& etc
    $lword =~ s/\\//g;

    # change all punctuation (except for . ^ _ ~, \protect as needed)
    # make sure to include all following space chars!
    $lword =~ s/([,;:\(\)\[\]\/\$\-|'`"+!?*=&]\s*)/##\1##/g;
    # _ and {} is valid code for ITRANS: a_i, n{}na etc, so left unchanged

    # change . ^ _ only if they are not part of ITRANS (^n ~n .n .C etc)
    $lword =~ s/([\.^~]+)([^iIcCnNdDtTrhaueo \t])/##\1##\2/g;
    $lword =~ s/([\.^~]+$)/##\1##/g;
    $lword =~ s/([\.^~]+\s+)/##\1##/g;

    # minimize: collapse multiple ##'s
    $lword =~ s/## ##/ /g;
    $lword =~ s/####//g;

    # HTML chars protections
    $lword =~ s/&/&amp;/g;

    return $lword;
}

#-------------------------------------------------------------------------
%STARTINDIAN = (
    "##", 1,
    "#indian", 2,
    "#hindi", 3,
    "#marathi", 4,
    "#sanskrit", 5,
    );
%ENDINDIAN = (
    "##", 1,
    "#endindian", 2,
    "#endhindi", 3,
    "#endmarathi", 4,
    "#endsanskrit", 5,
    );

%XFORMWORD = (
    "\\threedots", ". . .",
    "\\printtitle", "</PRE><PRE class=xdvng-eng>", # assume printtitle is outside #indian
    );

#-------------------------------------------------------------------------
$itransyes = 0; # whether parsing text that is itrans encoded

$startsong = 0;
$endsong = 0;

while (<>) {
    chop($line = $_);

    $startsong = 1 if $line =~ /\\startsong/;
    next if ! $startsong; # ignore all lines before \startsong
    $endsong = 1 if $line =~ /\\endsong/;
    last if $endsong; # ignore all lines after \endsong

    # erase certain kinds of comments/lines from the .s files
    $line =~ s/% ITRANS song.*//;
    $line =~ s/% ITRANS Song.*//;
    $line =~ s/% Begin indian text.*//;
    $line =~ s/% End indian text.*//;
    $line =~ s/^% send songs, corrections,.*$//;
    $line =~ s/^% ----.*//;
    $line =~ s/\\startsong//;
    $line =~ s/\\endsong//;

    # trim space at end
    $line =~ s/\s+$//;
    # remove last char if it is %
    $line =~ s/%+$//;

    next if $line eq "";

    # @words = split /\b/, $line;
    # first split - main objective is to get words like:
    # #indian #<word> \threedots \<word> ## (also #### should be two ##'s)
    @words = split /(##|#\w+|\\\w+)/, $line;

    $stitleseen = 0;
    $commentseen = 0;
    foreach $word (@words) {
	next if ! $word; # since the split /(..) results in empty words
	if ($word =~ /^\s*%/) {
	    # skip rest of words if this is a % line
	    $commentseen = 1;
	    $word =~ s/(^\s*)%/\1##%/ if ($itransyes); 
	}
	if ($commentseen) {
	    print $word;
	    next;
	}

	# \stitle lines contain ITRANS text, but it should be passed unchanged.
	# \stitle was used by TeX, so it allowed hindi/english text, for HTML
	# all ## directives should be disabled, so turn off shortmarkers.
	if ($word eq "\\stitle") {
	    $stitleseen = 1;
	    print "#ignoreshortmarkers";
	}
	if ($stitleseen) {
	    print $word;
	    next;
	}

        # print "found word :$word:\n";

	# first check if the entire word can be replaced
	$xword = $XFORMWORD{$word};
	if ($xword ne "") {
	    print "##" if $itransyes;
	    print $xword;
	    print "##" if $itransyes;
	    next; # word was replaced, all done
	}

	$xword = $word;

	# determine itransyes state
	$starti = $STARTINDIAN{$word};
	$endi = $ENDINDIAN{$word};
	if ($starti > 0 && $endi > 0) {
	    $itransyes = ! $itransyes;
	} elsif ($starti > 0) {
	    $itransyes = 1;
	} elsif ($endi > 0) {
	    $itransyes = 0;
	}

	# print "   itransyes: $itransyes\n" if $starti > 0 || $endi > 0;
	# next if $starti > 0 || $endi > 0;

	if ($itransyes) {

	    # call subroutine to make changes
	    $xword = &itxsubstitute($xword);

	    # print "    word xformed to:$xword\n" if $word ne $xword;
	} # if $itransyes

	print $xword;

    } # foreach $word

    if ($stitleseen == 1) {
	print "#useshortmarkers";
    }
    print "\n";
    if ($commentseen == 1) {
	# this has to come after the \n, since % ... \n is all comments
	print "##" if $itransyes;
    }
} # while (<>)

#-------------------------------------------------------------------------
#-------------------------------------------------------------------------
